@@ -34,7 +34,7 @@ CHECK_P3=0
 OPTIONS_HINTS=1
 
 # sets the GCC to use to the one required by the module (if available)
-function set_GCC_version () {
+set_GCC_version () {
   #identify GCC default version major number
   GCC_MAJOR="`gcc --version | grep -o -e "(GCC) ." | cut -d " " -f 2`"
 
@@ -222,8 +222,15 @@ else
   then
     kernel_release=`cat $src_file | grep UTS_RELEASE | cut -d'"' -f2`
   else
-    # UTS-define is in external version-*.h files, i.e. linux-2.2.14-5.0-RedHat does this flaw
-    kernel_release=`cat $linuxincludes/linux/version-*.h | grep UTS_RELEASE | grep \"$OsRelease\" | cut -d'"' -f2`
+    UTS_REL_COUNT=`cat $linuxincludes/linux/version-*.h 2>/dev/null | grep UTS_RELEASE -c`
+    if [ $UTS_REL_COUNT -gt 0 ];
+    then
+        # UTS-define is in external version-*.h files, i.e. linux-2.2.14-5.0-RedHat does this flaw
+        kernel_release=`cat $linuxincludes/linux/version-*.h | grep UTS_RELEASE | grep \"$OsRelease\" | cut -d'"' -f2`
+    else
+        # For 2.6.18 or higher, UTS-define is defined in utsrelease.h.
+        kernel_release=`cat $linuxincludes/linux/utsrelease.h | grep UTS_RELEASE | grep \"$OsRelease\" | cut -d'"' -f2`
+    fi
   fi
 fi
 
@@ -259,6 +266,18 @@ fi
 
 fi
 
+kernel_release_major=${kernel_release%%.*}
+kernel_release_rest=${kernel_release#*.}
+kernel_release_minor=${kernel_release_rest%%.*}
+if [ "$kernel_release_major" -lt 2 -o \
+    \( "$kernel_release_major" -eq 2 -a "$kernel_release_minor" -lt 6 \) ]; then
+    echo "Error:"
+    echo "Your kernel version $kernel_release is not supported by this driver release."
+    echo "Only 2.6.0 and newer kernels are supported."
+    exit 1
+fi
+
+
 OsVersion=${uname_v}
 
 if [ $DEMAND_BZIMAGE -gt 0 ]
@@ -441,13 +460,13 @@ else
   fi
 fi
 
-if [ $SMP = 0 ]
+if [ "$SMP" = 0 ]
 then
   echo "assuming default: SMP=$SMP"                                >> $logfile
 fi
 
 # act on final result
-if [ ! $SMP = 0 ]
+if [ ! "$SMP" = 0 ]
 then
   smp="-SMP"
   def_smp=-D__SMP__
@@ -606,7 +625,7 @@ then
     if [ -e ${lib_ip_base}.GCC$cc_version ];
     then
         # we do have an ip-lib that exactly matches the users compiler
-        ln -s ${lib_ip_base}.GCC$cc_version ${lib_ip_base}
+        ln -s ${lib_ip_base##*/}.GCC$cc_version ${lib_ip_base}
         echo "found exact match for ${CC} and the ip-library" >> $logfile
     else
         # there is no exact match for the users compiler
@@ -615,7 +634,7 @@ then
         do
             # just the last matching library does server our purposes - ease of coding
             rm -f ${lib_ip_base}
-            ln -s ${lib_ip_major} ${lib_ip_base}
+            ln -s ${lib_ip_major##*/} ${lib_ip_base}
         done
         
         # after the loop there should be a file or a symlink or whatever
@@ -719,7 +738,7 @@ def_for_all="-DATI_AGP_HOOK -DATI -DFGL -D${target_define} -DFGL_CUSTOM_MODULE -
 
 # defines for specific os and cpu platforms
 if [ "${uname_m}" = "x86_64" ]; then
-	def_machine="-mcmodel=kernel"
+	def_machine="-mcmodel=kernel -mno-red-zone"
 fi
 
 if [ "${uname_m}" = "ia64" ]; then
@@ -878,6 +897,49 @@ begin_end
   rm -f $tmp_src_file $tmp_obj_file $tmp_log_file 
 fi
 
+# =============================================================
+# Check if we're running a kernel version that has the RHEL 3u* implementation of do_munmap
+#
+if [ $kernel_is_26x -gt 0 ]
+then
+  ## skip for 2.6.x and higher
+  echo " Assuming default munmap API" | tee -a $logfile
+else
+  echo "Probing for munmap API version" | tee -a $logfile
+  # create a helper source file and try to compile it into an object file
+  tmp_src_file=tmp_munmap.c
+  tmp_obj_file=tmp_munmap.o
+  tmp_log_file=tmp_munmap.log
+# let's create the c file 
+  cat > $tmp_src_file <<-begin_end
+#define __KERNEL__
+#include <linux/mm.h>
+int probe_munmap_version(void) {
+    int addr=0;  
+    int size=0;	
+    do_munmap(current->mm,addr,size,0);
+    return 0;
+}
+begin_end
+ 
+  # check for vmap API compatibility
+  $CC -I$linuxincludes $tmp_src_file -c -o $tmp_obj_file &> $tmp_log_file
+  gcc_ret_munmap=$?
+    
+    
+# Check which MUNMAP API version it is and make define accordingly
+  def_munmap_api_version=
+  if [ $gcc_ret_munmap -eq 0 ]
+    then
+      echo "This is the munmap API used for RHEL 3u* "  | tee -a $logfile
+      def_munmap_api_version=-DFGL_LINUX_RHEL_MUNMAP_API
+    else
+      echo "Use default vmap API"                     | tee -a $logfile
+  fi
+  
+  # cleanup intermediate files
+  rm -f $tmp_src_file $tmp_obj_file $tmp_log_file 
+fi
 
 # ==============================================================
 # make agp kernel module (including object files) and check results
@@ -887,7 +949,11 @@ if [ $kernel_is_26x -gt 0 ]; then
     cd 2.6.x
     V=${V:-0}
     #tlog is a temporary file that will be deleted
-    make CC=${CC} V=${V} MODFLAGS="-DMODULE $def_for_all $def_smp $def_modversions" PAGE_ATTR_FIX=$PAGE_ATTR_FIX > tlog 2>&1 
+    echo '#This is a dummy file created to suppress the warning: < could not find /lib/modules/fglrx/build_mod/2.6.x/.libfglrx_ip.a.GCC4.cmd for /lib/modules/fglrx/build_mod/2.6.x/libfglrx_ip.a.GCC4 >' > .lib${MODULE}_ip.a.GCC${GCC_MAJOR}.cmd
+    make CC=${CC} V=${V} \
+	LIBIP_PREFIX=$(echo "$LIBIP_PREFIX" | sed -e 's|^\([^/]\)|../\1|') \
+	MODFLAGS="-DMODULE $def_for_all $def_smp $def_modversions" \
+	PAGE_ATTR_FIX=$PAGE_ATTR_FIX > tlog 2>&1 
     res=$?
     tee -a $logfile < tlog
     #delete tlog
@@ -946,87 +1012,13 @@ then
   fi
 fi
 
-#SRC=${SOURCE_PREFIX}/agpgart_fe.c
-#DST=agpgart_fe.o
-#echo "compiling '$SRC'..." | tee -a $logfile
-#cc_cmd="${CC} ${WARNINGS} -O2 -D__KERNEL__ -DMODULE -fomit-frame-pointer $def_for_all -D$MODULE $def_smp $def_modversions $patch_includes -I$linuxincludes -c $SRC -o $DST"
-#echo "$cc_cmd" >> $logfile
-#$cc_cmd 2>&1 | tee -a $logfile
-#if [ ! -e $DST ] ;
-#then
-#  echo "compiling failed - object file was not generated" | tee -a $logfile
-#  exit 1
-#fi
-
-SRC=${SOURCE_PREFIX}/agpgart_be.c
-DST=agpgart_be.o
-echo "compiling '$SRC'..." | tee -a $logfile
-cc_cmd="${CC} ${WARNINGS} -O2 -D__KERNEL__ -DMODULE -fomit-frame-pointer $def_for_all $def_machine -D$MODULE $def_smp $def_modversions $patch_includes -I$linuxincludes -c $SRC -o $DST"
-echo "$cc_cmd" >> $logfile
-$cc_cmd 2>&1 | tee -a $logfile
-if [ ! -e $DST ] ;
-then
-  echo "compiling failed - object file was not generated" | tee -a $logfile
-  exit 1
-fi
-
-SRC=${SOURCE_PREFIX}/agp3.c
-DST=agp3.o
-echo "compiling '$SRC'..." | tee -a $logfile
-cc_cmd="${CC} ${WARNINGS} -O2 -D__KERNEL__ -DMODULE -fomit-frame-pointer $def_for_all $def_machine -D$MODULE $def_smp $def_modversions $patch_includes -I$linuxincludes -c $SRC -o $DST"
-echo "$cc_cmd" >> $logfile
-$cc_cmd 2>&1 | tee -a $logfile
-if [ ! -e $DST ] ;
-then
-  echo "compiling failed - object file was not generated" | tee -a $logfile
-  exit 1
-fi
-
-SRC=${SOURCE_PREFIX}/i7505-agp.c
-DST=i7505-agp.o
-echo "compiling '$SRC'..." | tee -a $logfile
-cc_cmd="${CC} ${WARNINGS} -O2 -D__KERNEL__ -DMODULE -fomit-frame-pointer $def_for_all $def_machine -D$MODULE $def_smp $def_modversions $patch_includes -I$linuxincludes -c $SRC -o $DST"
-echo "$cc_cmd" >> $logfile
-$cc_cmd 2>&1 | tee -a $logfile
-if [ ! -e $DST ] ;
-then
-  echo "compiling failed - object file was not generated" | tee -a $logfile
-  exit 1
-fi
-
-SRC=${SOURCE_PREFIX}/nvidia-agp.c
-DST=nvidia-agp.o
-echo "compiling '$SRC'..." | tee -a $logfile
-cc_cmd="${CC} ${WARNINGS} -O2 -D__KERNEL__ -DMODULE -fomit-frame-pointer $def_for_all $def_machine -D$MODULE $def_smp $def_modversions $patch_includes -I$linuxincludes -c $SRC -o $DST"
-echo "$cc_cmd" >> $logfile
-$cc_cmd 2>&1 | tee -a $logfile
-if [ ! -e $DST ] ;
-then
-  echo "compiling failed - object file was not generated" | tee -a $logfile
-  exit 1
-fi
-
-# we don't need the agpgart module - skip that thing
-# echo "linking agp module..." | tee -a $logfile
-# ld="ld -r agpgart_fe.po agpgart_be.po -o agpgart.o"
-# echo "$ld" >> $logfile
-# $ld 2>&1 | tee -a $logfile
-# if [ ! -e ${MODULE}.o ] ;
-# then
-#  echo "linking failed - kernel module was not generated" | tee -a $logfile
-#  exit 1
-# fi
-#
-# echo .  >> $logfile
-
-
 # ==============================================================
 # make custom kernel module and check results
 
 SRC=${SOURCE_PREFIX}/${FGL_PUBLIC}_public.c
 DST=${FGL_PUBLIC}_public.o
 echo "compiling '$SRC'..." | tee -a $logfile
-cc_cmd="${CC} ${WARNINGS} -O2 -D__KERNEL__ -DMODULE -fomit-frame-pointer $def_for_all $def_machine -D${MODULE} $def_vma_api_version $def_vmap_api_version $def_smp $def_modversions $def_agp $patch_includes -I$linuxincludes -I$PWD -c $SRC -o $DST"
+cc_cmd="${CC} ${WARNINGS} -O2 -D__KERNEL__ -DMODULE -fomit-frame-pointer $def_for_all $def_machine -D${MODULE} $def_vma_api_version $def_vmap_api_version $def_munmap_api_version $def_smp $def_modversions $def_agp $patch_includes -I$linuxincludes -I$PWD -c $SRC -o $DST"
 echo "$cc_cmd" >> $logfile
 $cc_cmd 2>&1 | tee -a $logfile | grep -v "warning: pasting would not give a valid preprocessing token"
 if [ ! -e $DST ] ;
@@ -1039,7 +1031,7 @@ echo "linking of ${MODULE} kernel module..." | tee -a $logfile
 if [ ! -z "${MODULE_NAME}" ]; then 
 	module_version=.${MODULE_NAME}
 fi
-ld="ld -r ${FGL_PUBLIC}_public.o agpgart_be.o agp3.o i7505-agp.o nvidia-agp.o $core_lib -o ${MODULE}${module_version}.o"
+ld="ld -r ${FGL_PUBLIC}_public.o $core_lib -o ${MODULE}${module_version}.o"
 echo "$ld" >> $logfile
 $ld 2>&1 | tee -a $logfile
 if [ ! -e ${MODULE}${module_version}.o ] ;
@@ -1078,6 +1070,9 @@ then
 
   which strip > /dev/null 2>&1
   if test $? = 0; then
+     if [ ! -z "${FGLRX_DEBUG}" ]; then
+        cp -f ${MODULE}${kmod_extension} ${MODULE}_dbg${kmod_extension}
+     fi
      strip -g ${MODULE}${kmod_extension} > /dev/null 2>&1
      if test $? = 0; then	
         echo "stripping the debug info of kernel module" >> $logfile
