@@ -19,6 +19,15 @@ OS_MOD=/lib/modules
 # vendor options
 CHECK_P3=0
 
+# check if there was a custom specific values during install start
+if [ -f /etc/ati/inst_path_default -a -f /etc/ati/inst_path_override ]; then
+    . /etc/ati/inst_path_default
+    . /etc/ati/inst_path_override
+    if [ -n "${ATI_KERN_MOD}" ]; then
+        module=`basename ${ATI_KERN_MOD}`
+        OS_MOD=`dirname ${ATI_KERN_MOD}`
+    fi
+fi
 
 # ==============================================================
 # check if we are running as root with typical login shell paths
@@ -234,6 +243,12 @@ fi
             fi
         fi
     fi
+
+    # if there is a customized path for the kernel module, use that instead
+    if [ -n "${ATI_KERN_INST}" ]; then
+        TargetDir=${ATI_KERN_INST}
+    fi
+
 	TargetName=${module}${kmod_extension}
 	TargetFQN=${TargetDir}/${TargetName}
 	NonTarFQN=${NonTarDir}/${TargetName}
